import chalk     from 'chalk';
import stripAnsi from 'strip-ansi';
import supportsColor from 'supports-color';
import { regexes } from './regex.js';

const colorEnabled = supportsColor.stdout;

function clean(raw) {
  return stripAnsi(raw).replace(/\[\d+(;\d+)*m/g, '');
}

export function highlightLine(raw) {
  const line = clean(raw);

  if (!colorEnabled) {
    return line; // Si no hay soporte de color, devuelve la línea sin formatear
  }

  // 1) key: value (estilo util.inspect)
  const kv = line.match(/^(\s*)([A-Za-z0-9_$-]+)(:)\s*(.*)$/);
  if (kv) {
    const [ , indent, key, colon, rest ] = kv;
    const ck = chalk.cyan(key) + chalk.gray(colon + ' ');
    const cv = rest
      .replace(regexes.doubleQuotedString, (_, s) => chalk.green(`"${s}"`))
      .replace(regexes.singleQuotedString, (_, s) => chalk.green(`'${s}'`))
      .replace(/\b(true|false)\b/g,   m => chalk.magenta(m))
      .replace(/\b(null|undefined)\b/g, m => chalk.gray(m))
      .replace(regexes.time,           m => chalk.yellowBright(m))
      .replace(/\b\d+\b/g,             m => chalk.yellow(m))
      .replace(regexes.url,            m => chalk.cyan.underline(m))
      .replace(regexes.localhost,      m => chalk.cyanBright(m));
    return indent + ck + cv;
  }

  // 2) Checks y círculos
  if (/^\s*✓/.test(line)) {
    return chalk.green.bold(line);
  }
  if (/^\s*○/.test(line)) {
    return chalk.yellow.bold(line);
  }

  // 3) Palabras clave como "Compiling", "Starting", "Ready"
  if (regexes.keywordLine.test(line)) {
    return line.replace(regexes.keywordLine, m => chalk.blueBright.bold(m));
  }

  // 4) Resaltar "..."
  if (line.includes('...')) {
    return line.replace(/\.\.\./g, m => chalk.bold(m));
  }

  // 5) Métodos HTTP (al inicio)
  if (regexes.httpMethod.test(line)) {
    return line.replace(regexes.httpMethod, m => chalk.magentaBright.bold(m));
  }

  // 6) Niveles (“ERROR”, “WARN”, etc) al inicio
  if (regexes.logLevel.test(line)) {
    return line.replace(regexes.logLevel, m => {
      if (/ERROR|FATAL/i.test(m)) return chalk.red.bold(m);
      if (/WARN/i.test(m)) return chalk.yellow.bold(m);
      if (/INFO/i.test(m)) return chalk.blue.bold(m);
      if (/DEBUG|TRACE/i.test(m)) return chalk.gray.bold(m);
    });
  }

  // 7) Errores comunes de Java/Spring Boot
  if (regexes.javaError.test(line)) {
    return line.replace(regexes.javaError, m => chalk.red.bold(m));
  }

  // 8) Advertencias comunes de linters
  if (regexes.linterWarning.test(line)) {
    return line.replace(regexes.linterWarning, m => chalk.yellowBright.bold(m));
  }

  // 9) Frameworks en medio
  let out = line.replace(regexes.frameworks, m => chalk.magentaBright.bold(m));

  // 10) Versiones “x.y.z”
  out = out.replace(regexes.version, m => chalk.yellowBright(m));

  // 11) URLs y localhost
  out = out
    .replace(regexes.url,       m => chalk.cyan.underline(m))
    .replace(regexes.localhost, m => chalk.cyanBright(m));

  // 12) Tiempos, booleans, null, números sueltos, strings
  out = out
    .replace(regexes.time,       m => chalk.yellowBright(m))
    .replace(/\b(true|false)\b/g,   m => chalk.magenta(m))
    .replace(/\b(null|undefined)\b/g,m => chalk.gray(m))
    .replace(/\b\d+\b/g,            m => chalk.yellow(m))
    .replace(regexes.doubleQuotedString, (_, s) => chalk.green(`"${s}"`))
    .replace(regexes.singleQuotedString, (_, s) => chalk.green(`'${s}'`));

  return out;
}